@@ -6,7 +6,7 @@
 #ifndef _IP_VS_H
 #define _IP_VS_H
 
-#define IP_VS_VERSION_CODE            0x000208
+#define IP_VS_VERSION_CODE            0x000209
 #define NVERSION(version)                       \
         (version >> 16) & 0xFF,                 \
         (version >> 8) & 0xFF,                  \
@@ -257,7 +257,8 @@ struct ip_vs_conn {
 	struct list_head        c_list;         /* hashed list heads */
 	atomic_t                refcnt;         /* reference count */
 	struct timer_list       timer;          /* Expiration timer */
-	unsigned long           timeout;        /* timeout */
+
+        /* Protocol, addresses and port numbers */
         __u32                   caddr;          /* client address */
         __u32                   vaddr;          /* virtual address */
         __u32                   daddr;          /* destination address */
@@ -265,11 +266,17 @@ struct ip_vs_conn {
         __u16                   vport;
         __u16                   dport;
 	__u16                   protocol;       /* Which protocol (TCP/UDP) */
-	struct ip_vs_conn       *control;       /* Master control connection */
+
+        /* Flags and state transition */
+        spinlock_t              lock;           /* lock for state transition */
+	volatile __u16          flags;          /* status flags */
+	volatile __u16          state;          /* state info */
+	volatile unsigned long  timeout;        /* timeout */
+        struct ip_vs_timeout_table *timeout_table;
+
+	/* Control members */
+        struct ip_vs_conn       *control;       /* Master control connection */
 	atomic_t                n_control;      /* Number of controlled ones */
-	__u16                   flags;          /* status flags */
-	__u16                   state;          /* state info */
-	struct ip_vs_timeout_table *timeout_table;
 	struct ip_vs_dest       *dest;          /* real server */
         atomic_t                in_pkts;        /* incoming packet counter */
 
