@@ -1,7 +1,7 @@
 /*
  * IPVS         Application module
  *
- * Version:     $Id: ip_vs_app.c,v 1.10 2001/04/01 11:46:26 wensong Exp $
+ * Version:     $Id: ip_vs_app.c,v 1.11 2001/04/12 02:59:31 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -283,12 +283,15 @@ vs_fix_ack_seq(const struct ip_vs_seq *vseq, struct tcphdr *th)
 static inline void vs_seq_update(struct ip_vs_conn *cp, struct ip_vs_seq *vseq,
                                  unsigned flag, __u32 seq, int diff)
 {
+        /* spinlock is to keep updating cp->flags atomic */
+        spin_lock(&cp->lock);
         if ( !(cp->flags & flag) || after(seq, vseq->init_seq)) {
                 vseq->previous_delta = vseq->delta;
                 vseq->delta += diff;
                 vseq->init_seq = seq;
                 cp->flags |= flag;
         }
+        spin_unlock(&cp->lock);
 }
 
 
