@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.14 2001/04/04 02:53:11 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.15 2001/04/12 02:59:31 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -505,7 +505,6 @@ static inline int vs_tcp_state_idx(struct tcphdr *th, int state_off)
 static inline int vs_set_state_timeout(struct ip_vs_conn *cp, int state)
 {
 	struct ip_vs_timeout_table *vstim = cp->timeout_table;
-	int scale;
 
 	/*
 	 *	Use default timeout table if no specific for this entry
@@ -514,12 +513,15 @@ static inline int vs_set_state_timeout(struct ip_vs_conn *cp, int state)
 		vstim = &vs_timeout_table;
 
 	cp->timeout = vstim->timeout[cp->state=state];
-	scale = vstim->scale;
 
-	if (scale<0)
-		cp->timeout >>= -scale;
-	else if (scale > 0)
-		cp->timeout <<= scale;
+        if (vstim->scale) {
+                int scale = vstim->scale;
+
+                if (scale<0)
+                        cp->timeout >>= -scale;
+                else if (scale > 0)
+                        cp->timeout <<= scale;
+        }
 
 	return state;
 }
@@ -592,15 +594,22 @@ vs_tcp_state(struct ip_vs_conn *cp, int state_off, struct tcphdr *th)
 int ip_vs_set_state(struct ip_vs_conn *cp,
                     int state_off, struct iphdr *iph, void *tp)
 {
-	switch (iph->protocol) {
-		case IPPROTO_ICMP:
-			return vs_set_state_timeout(cp, IP_VS_S_ICMP);
-		case IPPROTO_UDP:
-			return vs_set_state_timeout(cp, IP_VS_S_UDP);
-		case IPPROTO_TCP:
-			return vs_tcp_state(cp, state_off, tp);
-	}
-	return -1;
+        int ret;
+
+        spin_lock(&cp->lock);
+        switch (iph->protocol) {
+        case IPPROTO_ICMP:
+                ret = vs_set_state_timeout(cp, IP_VS_S_ICMP);
+        case IPPROTO_UDP:
+                ret = vs_set_state_timeout(cp, IP_VS_S_UDP);
+        case IPPROTO_TCP:
+                ret = vs_tcp_state(cp, state_off, tp);
+        default:
+                ret = -1;
+        }
+        spin_unlock(&cp->lock);
+
+	return ret;
 }
 
 
@@ -1207,60 +1216,58 @@ static inline void ip_vs_timeout_detach(struct ip_vs_conn *cp)
 
 static void ip_vs_conn_expire(unsigned long data)
 {
-	struct ip_vs_conn *cp = (struct ip_vs_conn *)data;
+        struct ip_vs_conn *cp = (struct ip_vs_conn *)data;
 
-	if (cp->timeout_table)
+        if (cp->timeout_table)
                 cp->timeout = cp->timeout_table->timeout[IP_VS_S_TIME_WAIT];
         else
                 cp->timeout = vs_timeout_table.timeout[IP_VS_S_TIME_WAIT];
 
-	/*
-	 *	hey, I'm using it
-	 */
-	atomic_inc(&cp->refcnt);
+        /*
+         *	hey, I'm using it
+         */
+        atomic_inc(&cp->refcnt);
 
-	/*
-	 * 	do I control anybody?
-	 */
-	if (atomic_read(&cp->n_control)) 
-		goto expire_later;
+        /*
+         * 	do I control anybody?
+         */
+        if (atomic_read(&cp->n_control)) 
+                goto expire_later;
 
-	/*
-	 *	does anybody controls me?
-	 */
-	if (cp->control)
-		ip_vs_control_del(cp);
-
-	/*
-	 *	unhash it if it is hashed in the conn table
-	 */
+        /*
+         *	unhash it if it is hashed in the conn table
+         */
         ip_vs_conn_unhash(cp);
 
-	/*
-	 *	refcnt==1 implies I'm the only one referrer
-	 */
-	if (atomic_read(&cp->refcnt) == 1) {
+        /*
+         *	refcnt==1 implies I'm the only one referrer
+         */
+        if (atomic_read(&cp->refcnt) == 1) {
                 /* make sure that there is no timer on it now */
                 if (timer_pending(&cp->timer))
                         del_sltimer(&cp->timer);
 
+                /* does anybody control me? */
+                if (cp->control)
+                        ip_vs_control_del(cp);
+
                 ip_vs_unbind_dest(cp);
                 ip_vs_unbind_app(cp);
-		ip_vs_timeout_detach(cp);
+                ip_vs_timeout_detach(cp);
                 if (cp->flags & IP_VS_CONN_F_NO_CPORT)
                         atomic_dec(&ip_vs_conn_no_cport_cnt);
-		kmem_cache_free(ip_vs_conn_cachep, cp);
-		/*  sysctl_ip_always_defrag--; */
-		MOD_DEC_USE_COUNT;
-		goto expire_out;
-	}
+                kmem_cache_free(ip_vs_conn_cachep, cp);
+                /*  sysctl_ip_always_defrag--; */
+                MOD_DEC_USE_COUNT;
+                goto expire_out;
+        }
 
   expire_later:
-	IP_VS_DBG(7, "delayed: refcnt-1=%d conn.n_control=%d\n",
+        IP_VS_DBG(7, "delayed: refcnt-1=%d conn.n_control=%d\n",
                   atomic_read(&cp->refcnt)-1,
                   atomic_read(&cp->n_control));
 
-	ip_vs_conn_put(cp);
+        ip_vs_conn_put(cp);
 
   expire_out:
 }
@@ -1299,10 +1306,15 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
         cp->flags	   = flags;
         cp->app_data	   = NULL;
         cp->control	   = NULL;
+        cp->lock           = SPIN_LOCK_UNLOCKED;
 
 	atomic_set(&cp->n_control, 0);
 	atomic_set(&cp->in_pkts, 0);
 
+        /* Bind its app if there is one, and set its state and timeout */
+        ip_vs_bind_app(cp);
+        vs_set_state_timeout(cp, IP_VS_S_NONE);
+
         /*
          * Set the entry is referenced by the current thread before hashing
          * it in the table, so that other thread run ip_vs_random_dropentry
@@ -1310,15 +1322,12 @@ ip_vs_conn_new(int proto, __u32 caddr, __u16 cport, __u32 vaddr, __u16 vport,
          */
 	atomic_set(&cp->refcnt, 1);
 
-        /*  Hash it in the ip_vs_conn_tab */
+        /* Hash it in the ip_vs_conn_tab finally */
         ip_vs_conn_hash(cp);
 
         if (flags & IP_VS_CONN_F_NO_CPORT)
                 atomic_inc(&ip_vs_conn_no_cport_cnt);
 
-        ip_vs_bind_app(cp);
-        vs_set_state_timeout(cp, IP_VS_S_NONE);
-
         return cp;
 }
 
@@ -1395,7 +1404,9 @@ static inline int todrop_entry(struct ip_vs_conn *cp)
 	static char todrop_counter[9] = {0};
 	int i;
 
-	/* !!! TODO: cp->timeout locking */
+        /* if the conn entry hasn't lasted for 60 seconds, don't drop it.
+           This will leave enough time for normal connection to get
+           through. */
 	if (cp->timeout+jiffies-cp->timer.expires < 60*HZ)
 		return 0;
 
@@ -1457,15 +1468,16 @@ void ip_vs_random_dropentry(void)
 			 */
 			ct_write_unlock(hash);
 			IP_VS_DBG(4, "Drop connection\n");
-			ct = cp->control;
 			fn = (cp->timer).function;
-			if (del_sltimer(&cp->timer))
+			if (del_sltimer(&cp->timer)) {
+                                ct = cp->control;
 				fn((unsigned long)cp);
-			if (ct && !atomic_read(&ct->n_control)) {
-				IP_VS_DBG(4, "Drop connection template\n");
-				if(del_sltimer(&ct->timer))
-                                        fn((unsigned long)ct);
-			}
+                                if (ct && !atomic_read(&ct->n_control)) {
+                                        IP_VS_DBG(4, "Drop conn template\n");
+                                        if(del_sltimer(&ct->timer))
+                                                fn((unsigned long)ct);
+                                }
+                        }
 			ct_write_lock(hash);
 		}
 		ct_write_unlock(hash);
