@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.14 2001/03/18 15:01:45 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.15 2001/03/22 12:57:46 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -48,7 +48,7 @@
 static spinlock_t __ip_vs_mutex = SPIN_LOCK_UNLOCKED;
 
 /* lock for service table */
-static rwlock_t __ip_vs_svc_lock = RW_LOCK_UNLOCKED;
+rwlock_t __ip_vs_svc_lock = RW_LOCK_UNLOCKED;
 
 /* lock for table with the real services */
 static rwlock_t __ip_vs_rs_lock = RW_LOCK_UNLOCKED;
@@ -339,8 +339,7 @@ __ip_vs_service_get(__u16 protocol, __u32 vaddr, __u16 vport)
 /*
  *  Get service by {fwmark} in the service table.
  */
-static __inline__ struct ip_vs_service *
-__ip_vs_svc_fwm_get(unsigned long fwmark)
+static __inline__ struct ip_vs_service *__ip_vs_svc_fwm_get(__u32 fwmark)
 {
 	unsigned hash;
 	struct ip_vs_service *svc;
@@ -365,8 +364,7 @@ __ip_vs_svc_fwm_get(unsigned long fwmark)
 }
 
 struct ip_vs_service *
-ip_vs_service_get(unsigned long fwmark,
-                  __u16 protocol, __u32 vaddr, __u16 vport)
+ip_vs_service_get(__u32 fwmark, __u16 protocol, __u32 vaddr, __u16 vport)
 {
 	struct ip_vs_service *svc;
 
@@ -409,7 +407,7 @@ ip_vs_service_get(unsigned long fwmark,
   out:
 	read_unlock(&__ip_vs_svc_lock);
 
-        IP_VS_DBG(6, "lookup service: fwm %ld %s %u.%u.%u.%u:%u %s\n",
+        IP_VS_DBG(6, "lookup service: fwm %u %s %u.%u.%u.%u:%u %s\n",
                   fwmark, ip_vs_proto_name(protocol),
                   NIPQUAD(vaddr), ntohs(vport),
                   svc?"hit":"not hit");
@@ -554,7 +552,7 @@ ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 
 	for (e=l->next; e!=l; e=e->next) {
 		dest = list_entry(e, struct ip_vs_dest, n_list);
-		IP_VS_DBG(3, "Destination %lu/%u.%u.%u.%u:%u still in trash, "
+		IP_VS_DBG(3, "Destination %u/%u.%u.%u.%u:%u still in trash, "
                           "refcnt=%d\n",
                           dest->vfwmark,
                           NIPQUAD(dest->addr), ntohs(dest->port),
@@ -574,7 +572,7 @@ ip_vs_get_trash_dest(struct ip_vs_service *svc, __u32 daddr, __u16 dport)
 		 * Try to purge the destination from trash if not referenced
 		 */
 		if (atomic_read(&dest->refcnt) == 1) {
-			IP_VS_DBG(3, "Removing destination %lu/%u.%u.%u.%u:%u "
+			IP_VS_DBG(3, "Removing destination %u/%u.%u.%u.%u:%u "
                                   "from trash\n",
 				  dest->vfwmark,
 				  NIPQUAD(dest->addr), ntohs(dest->port));
@@ -723,7 +721,7 @@ static int ip_vs_add_dest(struct ip_vs_service *svc,
 	dest = ip_vs_get_trash_dest(svc, daddr, dport);
 	if (dest != NULL) {
 		IP_VS_DBG(3, "Get destination %u.%u.%u.%u:%u from trash, "
-                          "refcnt=%d, service %lu/%u.%u.%u.%u:%u\n",
+                          "refcnt=%d, service %u/%u.%u.%u.%u:%u\n",
                           NIPQUAD(daddr), ntohs(dport),
                           atomic_read(&dest->refcnt),
                           dest->vfwmark,
@@ -1003,7 +1001,8 @@ ip_vs_add_service(struct ip_vs_rule_user *ur, struct ip_vs_service **svc_p)
 	return 0;
 
   out_err:
-	__MOD_DEC_USE_COUNT(sched->module);
+        if (sched->module)
+                __MOD_DEC_USE_COUNT(sched->module);
 	MOD_DEC_USE_COUNT;
 	if (svc) {
 		kfree(svc);
@@ -1078,7 +1077,7 @@ static int ip_vs_edit_service(struct ip_vs_service *svc,
   out:
         write_unlock_bh(&__ip_vs_svc_lock);
 
-        if (old_sched)
+        if (old_sched && old_sched->module)
                 __MOD_DEC_USE_COUNT(old_sched->module);
 
         return ret;
@@ -1100,7 +1099,7 @@ static int __ip_vs_del_service(struct ip_vs_service *svc)
          */
         old_sched = svc->scheduler;
         ip_vs_unbind_scheduler(svc);
-        if (old_sched)
+        if (old_sched && old_sched->module)
                 __MOD_DEC_USE_COUNT(old_sched->module);
 
         /*
@@ -1353,8 +1352,8 @@ static int ip_vs_get_info(char *buf, char **start, off_t offset, int length)
 
         /*
          * Note: since the length of the buffer is usually the multiple
-         * of 512, it is easier and faster to use fixed record of the
-         * divisor of 512, in order to avoid buffer overflow. 
+         * of 512, it is good to use fixed record of the divisor of 512,
+         * so that records won't be truncated at buffer boundary.
          */
         pos = 192;
         if (pos > offset) {
@@ -1378,13 +1377,13 @@ static int ip_vs_get_info(char *buf, char **start, off_t offset, int length)
                         pos += 64;
                         if (pos > offset) {
                                 if (svc->flags & IP_VS_SVC_F_PERSISTENT)
-                                        sprintf(temp2, " persistent %d %08X\n",
+                                        sprintf(temp2, "persistent %d %08X",
                                                 svc->timeout,
                                                 ntohl(svc->netmask));
                                 else
-                                        sprintf(temp2, "\n");
+                                        temp2[0] = '\0';
                                 
-                                sprintf(temp, "%s  %08X:%04X %s",
+                                sprintf(temp, "%s  %08X:%04X %s %s",
                                         ip_vs_proto_name(svc->protocol),
                                         ntohl(svc->addr),
                                         ntohs(svc->port),
@@ -1423,11 +1422,11 @@ static int ip_vs_get_info(char *buf, char **start, off_t offset, int length)
                         pos += 64;
                         if (pos > offset) {
                                 if (svc->flags & IP_VS_SVC_F_PERSISTENT)
-                                        sprintf(temp2, " persistent %d %08X\n",
+                                        sprintf(temp2, "persistent %d %08X",
                                                 svc->timeout,
                                                 ntohl(svc->netmask));
                                 else
-                                        sprintf(temp2, "\n");
+                                        temp2[0] = '\0';
                                 
                                 sprintf(temp, "FWM  %08X %s %s",
                                         svc->fwmark,
@@ -1487,15 +1486,17 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
         pos += 128;
         if (pos > offset) {
                 len += sprintf(buf+len, "%-63s\n",
-/*                              0123456789 0123456789 0123456789 0123456789 0123456789 */
-                               "TotalConns     InPkts    OutPkts    InBytes   OutBytes");
+/*                                01234567 01234567 01234567 0123456701234567 0123456701234567 */
+                               "TotalConns   InPkts  OutPkts          InBytes         OutBytes");
                 spin_lock(&ip_vs_stats.lock);
-                sprintf(temp, "%10lu %10lu %10lu %10lu %10lu",
+                sprintf(temp, "  %8X %8X %8X %8X%08X %8X%08X",
                         ip_vs_stats.conns,
                         ip_vs_stats.inpkts,
                         ip_vs_stats.outpkts,
-                        ip_vs_stats.inbytes,
-                        ip_vs_stats.outbytes);
+                        (__u32)(ip_vs_stats.inbytes >> 32),
+                        (__u32)ip_vs_stats.inbytes,
+                        (__u32)(ip_vs_stats.outbytes >> 32),
+                        (__u32)ip_vs_stats.outbytes);
                 spin_unlock(&ip_vs_stats.lock);
                 len += sprintf(buf+len, "%-63s\n", temp);
         }
@@ -1503,36 +1504,36 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
         read_lock_bh(&__ip_vs_svc_lock);
         
         /* print the service statistics */
-        pos += 90;
+        pos += 128;
         if (pos > offset) {
-                len += sprintf(buf+len, "%-89s\n",
+                len += sprintf(buf+len, "%-127s\n",
                                "\nVirtual Service\n"
-                               "Pro VirtService        Conns     InPkts    OutPkts    InBytes   OutBytes");
+                               "Pro VirtService      Conns   InPkts  OutPkts          InBytes         OutBytes");
         }
         
         for (idx = 0; idx < IP_VS_SVC_TAB_SIZE; idx++) {
                 l = &ip_vs_svc_table[idx];
                 for (e=l->next; e!=l; e=e->next) {
                         svc = list_entry(e, struct ip_vs_service, s_list);
-                        pos += 80;
+                        pos += 128;
                         if (pos <= offset)
                                 continue;
-                        if (pos > offset+length) {
-                                pos -= 80;
-                                goto done;
-                        }
                         spin_lock(&svc->stats.lock);
-                        sprintf(temp, "%3s %08X:%04X %10lu %10lu %10lu %10lu %10lu",
+                        sprintf(temp, "%3s %08X:%04X %8X %8X %8X %8X%08X %8X%08X",
                                 ip_vs_proto_name(svc->protocol),
                                 ntohl(svc->addr),
                                 ntohs(svc->port),
                                 svc->stats.conns,
                                 svc->stats.inpkts,
                                 svc->stats.outpkts,
-                                svc->stats.inbytes,
-                                svc->stats.outbytes);
+                                (__u32)(svc->stats.inbytes >> 32),
+                                (__u32)svc->stats.inbytes,
+                                (__u32)(svc->stats.outbytes >> 32),
+                                (__u32)svc->stats.outbytes);
                         spin_unlock(&svc->stats.lock);
-                        len += sprintf(buf+len, "%-79s\n", temp);
+                        len += sprintf(buf+len, "%-127s\n", temp);
+                        if (pos >= offset+length)
+                                goto done;
                 }
         }
 
@@ -1540,36 +1541,32 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
                 l = &ip_vs_svc_fwm_table[idx];
                 for (e=l->next; e!=l; e=e->next) {
                         svc = list_entry(e, struct ip_vs_service, f_list);
-                        pos += 80;
+                        pos += 128;
                         if (pos <= offset)
                                 continue;
-                        if (pos > offset+length) {
-                                pos -= 80;
-                                goto done;
-                        }
                         spin_lock(&svc->stats.lock);
-                        sprintf(temp, "FWM %-13X %10lu %10lu %10lu %10lu %10lu",
+                        sprintf(temp, "FWM %08X      %8X %8X %8X %8X%08X %8X%08X",
                                 svc->fwmark,
                                 svc->stats.conns,
                                 svc->stats.inpkts,
                                 svc->stats.outpkts,
-                                svc->stats.inbytes,
-                                svc->stats.outbytes);
+                                (__u32)(svc->stats.inbytes >> 32),
+                                (__u32)svc->stats.inbytes,
+                                (__u32)(svc->stats.outbytes >> 32),
+                                (__u32)svc->stats.outbytes);
                         spin_unlock(&svc->stats.lock);
-                        len += sprintf(buf+len, "%-79s\n", temp);
+                        len += sprintf(buf+len, "%-127s\n", temp);
+                        if (pos >= offset+length)
+                                goto done;
                 }
         }
 
         /* print the real server statistics */
-        pos += 105;
-        if (pos > offset+length) {
-                pos -= 105;
-                goto done;
-        }
+        pos += 128;
         if (pos > offset) {
-                len += sprintf(buf+len, "%-104s\n",
+                len += sprintf(buf+len, "%-127s\n",
                                "\nReal Service\n"
-                               "Pro VirtService   RealService        Conns     InPkts    OutPkts    InBytes   OutBytes");
+                               "Pro VirtService   RealService      Conns   InPkts  OutPkts          InBytes         OutBytes");
         }
         
         for (idx = 0; idx < IP_VS_SVC_TAB_SIZE; idx++) {
@@ -1579,16 +1576,12 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
                         p = &svc->destinations;
                         for (q=p->next; q!=p; q=q->next) {
                                 dest = list_entry(q, struct ip_vs_dest, n_list);
-                                pos += 90;
+                                pos += 128;
                                 if (pos <= offset)
                                         continue;
-                                if (pos > offset+length) {
-                                        pos -= 90;
-                                        goto done;
-                                }
                                 spin_lock(&dest->stats.lock);
                                 sprintf(temp,
-                                        "%3s %08X:%04X %08X:%04X %10lu %10lu %10lu %10lu %10lu",
+                                        "%3s %08X:%04X %08X:%04X %8X %8X %8X %8X%08X %8X%08X",
                                         ip_vs_proto_name(svc->protocol),
                                         ntohl(svc->addr),
                                         ntohs(svc->port),
@@ -1597,10 +1590,14 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
                                         dest->stats.conns,
                                         dest->stats.inpkts,
                                         dest->stats.outpkts,
-                                        dest->stats.inbytes,
-                                        dest->stats.outbytes);
+                                        (__u32)(dest->stats.inbytes >> 32),
+                                        (__u32)dest->stats.inbytes,
+                                        (__u32)(dest->stats.outbytes >> 32),
+                                        (__u32)dest->stats.outbytes);
                                 spin_unlock(&dest->stats.lock);
-                                len += sprintf(buf+len, "%-89s\n", temp);
+                                len += sprintf(buf+len, "%-127s\n", temp);
+                                if (pos >= offset+length)
+                                        goto done;
                         }
                 }
         }
@@ -1612,26 +1609,26 @@ ip_vs_stats_get_info(char *buf, char **start, off_t offset, int length)
                         p = &svc->destinations;
                         for (q=p->next; q!=p; q=q->next) {
                                 dest = list_entry(q,struct ip_vs_dest,n_list);
-                                pos += 90;
+                                pos += 128;
                                 if (pos <= offset)
                                         continue;
-                                if (pos > offset+length) {
-                                        pos -= 90;
-                                        goto done;
-                                }
                                 spin_lock(&dest->stats.lock);
                                 sprintf(temp,
-                                        "FWM %-13X %08X:%04X %10lu %10lu %10lu %10lu %10lu",
+                                        "FWM %08X      %08X:%04X %8X %8X %8X %8X%08X %8X%08X",
                                         svc->fwmark,
                                         ntohl(dest->addr),
                                         ntohs(dest->port),
                                         dest->stats.conns,
                                         dest->stats.inpkts,
                                         dest->stats.outpkts,
-                                        dest->stats.inbytes,
-                                        dest->stats.outbytes);
+                                        (__u32)(dest->stats.inbytes >> 32),
+                                        (__u32)dest->stats.inbytes,
+                                        (__u32)(dest->stats.outbytes >> 32),
+                                        (__u32)dest->stats.outbytes);
                                 spin_unlock(&dest->stats.lock);
-                                len += sprintf(buf+len, "%-89s\n", temp);
+                                len += sprintf(buf+len, "%-127s\n", temp);
+                                if (pos >= offset+length)
+                                        goto done;
                         }
                 }
         }
