@@ -1,7 +1,7 @@
 /*
  * IPVS:        Locality-Based Least-Connection with Replication scheduler
  *
- * Version:     $Id: ip_vs_lblcr.c,v 1.4 2001/01/29 15:27:27 wensong Exp $
+ * Version:     $Id: ip_vs_lblcr.c,v 1.5 2001/02/15 15:12:08 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@gnuchina.org>
  *
@@ -23,7 +23,8 @@
  *               n, serverSet[dest_ip] <- {weighted least-conn node};
  *       else
  *               n <- {least-conn (alive) node in serverSet[dest_ip]};
- *               if (n.conns>n.weight AND 
+ *               if (n is null) OR
+ *                  (n.conns>n.weight AND 
  *                   there is a node m with m.conns<m.weight/2) then
  *                   n <- {weighted least-conn node};
  *                   add n to serverSet[dest_ip];
@@ -821,9 +822,7 @@ ip_vs_lblcr_schedule(struct ip_vs_service *svc, struct iphdr *iph)
                 ip_vs_lblcr_hash(tbl, en);
         } else {
                 dest = ip_vs_dest_set_min(&en->set);
-		if (!dest)
-			return NULL;
-                if (is_overloaded(dest, svc)) {
+                if (!dest || is_overloaded(dest, svc)) {
                         dest = __ip_vs_wlc_schedule(svc, iph);
                         if (dest == NULL) {
                                 IP_VS_DBG(1, "no destination available\n");
