@@ -5,9 +5,9 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_timer.c,v 1.3 2000/07/06 14:57:09 wensong Exp $
+ * Version:     $Id: ip_vs_timer.c,v 1.4 2000/09/06 08:36:36 wensong Exp $
  *
- * Authors:     Wensong Zhang <wensong@iinchina.net>
+ * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Julian Anastasov <uli@linux.tu-varna.acad.bg>
  *
  *              This program is free software; you can redistribute it and/or
@@ -208,7 +208,6 @@ static inline void run_sltimer_list(void)
 
 			detach_sltimer(timer);
 			timer->list.next = timer->list.prev = NULL;
-			timer_set_running(timer);
 			write_unlock(&__ip_vs_sltimerlist_lock);
 			fn(data);
 			write_lock(&__ip_vs_sltimerlist_lock);
